#ifndef POKEPLATINUM_UNK_0202D7A8_H
#define POKEPLATINUM_UNK_0202D7A8_H

#include "struct_defs/struct_0202D7B0.h"
#include "struct_defs/struct_0202D844.h"
#include "struct_defs/struct_0202D84C.h"
#include "struct_defs/struct_020698E4.h"
#include "struct_defs/struct_0206C638.h"
#include "struct_decls/struct_021C0794_decl.h"

int sub_0202D7A8(void);
void sub_0202D7B0(UnkStruct_0202D7B0 * param0);
void sub_0202D80C(UnkStruct_0202D7B0 * param0, const u32 param1);
u32 sub_0202D814(UnkStruct_0202D7B0 * param0, const u8 param1);
UnkStruct_020698E4 * sub_0202D830(UnkStruct_0202D7B0 * param0);
UnkStruct_0202D7B0 * sub_0202D834(UnkStruct_021C0794 * param0);
UnkStruct_0202D844 * sub_0202D840(UnkStruct_0202D7B0 * param0);
const int sub_0202D844(UnkStruct_0202D844 * param0);
void sub_0202D848(const u8 param0, UnkStruct_0202D844 * param1);
UnkStruct_0202D84C * sub_0202D84C(const u8 param0, UnkStruct_0202D844 * param1);
void sub_0202D854(UnkStruct_021C0794 * param0, const int param1);
void sub_0202D884(UnkStruct_021C0794 * param0);
u8 sub_0202D898(UnkStruct_0202D7B0 * param0);
void sub_0202D8A4(UnkStruct_0202D7B0 * param0, const int param1);
int sub_0202D8BC(UnkStruct_0202D7B0 * param0);
u8 sub_0202D8C4(UnkStruct_0202D7B0 * param0, const u8 param1);
void sub_0202D8DC(UnkStruct_0202D7B0 * param0, const u8 param1, const u8 param2);
u8 sub_0202D8F8(UnkStruct_0202D7B0 * param0, const u8 param1);
void sub_0202D914(UnkStruct_0206C638 ** param0);
UnkStruct_0206C638 * sub_0202D924(UnkStruct_0202D7B0 * param0, const u8 param1);
u32 sub_0202D93C(const UnkStruct_0206C638 * param0, const u8 param1);
void sub_0202D980(UnkStruct_0206C638 * param0, const u8 param1, const u32 param2);
u8 * sub_0202D9C4(UnkStruct_0202D7B0 * param0);
u8 * sub_0202D9CC(UnkStruct_0202D7B0 * param0);
BOOL sub_0202D9D8(UnkStruct_0202D7B0 * param0);
void sub_0202D9EC(UnkStruct_0202D7B0 * param0, const u8 param1);
u8 sub_0202DA04(UnkStruct_0202D7B0 * param0);
void sub_0202DA10(UnkStruct_021C0794 * param0, u16 * param1, u16 * param2);
void sub_0202DA24(UnkStruct_021C0794 * param0, const u16 param1);

#endif // POKEPLATINUM_UNK_0202D7A8_H
